import re

def clear(text):
    txt = re.sub("\033\[[0-9][0-9]?m", "", text)
    txt = re.sub("\\u001b\[[0-9][0-9]?m", "", text)
    txt = re.sub("\\u001b\[[0-9][0-9]?;1m", "", text)
    return txt

def black(text): return '\033[30m' + text + '\033[0m'
def red(text): return '\033[31m' + text + '\033[0m'
def green(text): return '\033[32m' + text + '\033[0m'
def yellow(text): return '\033[33m' + text + '\033[0m'
def blue(text): return '\033[34m' + text + '\033[0m'
def purple(text): return '\033[35m' + text + '\033[0m'
def cyan(text): return '\033[36m' + text + '\033[0m'
def lightgrey(text): return '\033[37m' + text + '\033[0m'
def darkgrey(text): return '\033[90m' + text + '\033[0m'
def lightred(text): return '\033[91m' + text + '\033[0m'
def lightgreen(text): return '\033[92m' + text + '\033[0m'
def yellow(text): return '\033[93m' + text + '\033[0m'
def lightblue(text): return '\033[94m' + text + '\033[0m'
def pink(text): return '\033[95m' + text + '\033[0m'
def lightcyan(text): return '\033[96m' + text + '\033[0m'

def bold(text): return '\033[01m' + text + '\033[0m'
def disable(text): return '\033[02m' + text + '\033[0m'
def underline(text): return '\033[04m' + text + '\033[0m'
def reverse(text): return '\033[07m' + text + '\033[0m'
def strikethrough(text): return '\033[09m' + text + '\033[0m'
def invisible(text): return '\033[08m' + text + '\033[0m'

def on_black(text): return '\033[40m' + text + '\033[0m'
def on_red(text): return '\033[41m' + text + '\033[0m'
def on_green(text): return '\033[42m' + text + '\033[0m'
def on_yellow(text): return '\033[43m' + text + '\033[0m'
def on_blue(text): return '\033[44m' + text + '\033[0m'
def on_purple(text): return '\033[45m' + text + '\033[0m'
def on_cyan(text): return '\033[46m' + text + '\033[0m'
def on_lightgrey(text): return '\033[47m' + text + '\033[0m'

stop = '\033[0m'
class fg:
    black = '\033[30m'
    red = '\033[31m'
    green = '\033[32m'
    orange = '\033[33m'
    blue = '\033[34m'
    purple = '\033[35m'
    cyan = '\033[36m'
    lightgrey = '\033[37m'
    darkgrey = '\033[90m'
    lightred = '\033[91m'
    lightgreen = '\033[92m'
    yellow = '\033[93m'
    lightblue = '\033[94m'
    pink = '\033[95m'
    lightcyan = '\033[96m'

class bg:
    black = '\033[40m'
    red = '\033[41m'
    green = '\033[42m'
    orange = '\033[43m'
    blue = '\033[44m'
    purple = '\033[45m'
    cyan = '\033[46m'
    lightgrey = '\033[47m'


from . import color
from time import time
import datetime

class Bar():
    def __init__(self, max = 1, width = 80, prefix = "", eta = True, decimals = 0, duration = True):
        self.max = max
        self.width = width if type(width) == int else 80
        self.prefix = prefix
        self.eta = eta
        self.decimals = decimals
        self.lastProgress = 0
        self.lastUpdate = None
        self.lastETA = "-"
        self.start_at = time()
        self.duration = duration

    def update_eta(self, progress):
        if self.lastUpdate is not None and time()-self.lastUpdate < 1: return self.lastETA

        progression = progress - self.lastProgress
        left = (self.max-progress)

        if progression == 0 or self.lastUpdate is None:
            self.lastUpdate = time()
            self.lastProgress = progress
            self.lastETA = "-"
            return self.lastETA

        seconds = left/progression * (time() - self.lastUpdate)
        self.lastUpdate = time()
        self.lastProgress = progress
        if seconds < 0: self.lastETA = "-"
        else: self.lastETA = str(datetime.timedelta(seconds=seconds)).split(".")[0]
        return self.lastETA

    def __call__(self, progress: float, prefix=None, stop=False):
        if prefix is None: prefix = self.prefix
        else: prefix = str(prefix)

        progress_normed = progress / self.max
        if progress == self.max : stop = True
        c = color.fg.yellow if stop and progress_normed != 1 else color.fg.lightgreen 

        if stop: end = "\n"
        else:    end = "\r"

        if type(progress) == float: progress = round(progress, self.decimals)
        percent  = f" {c}{round(progress_normed*100,self.decimals)                                        if self.decimals > 0 else int(progress_normed*100)}%"
        frac     = f" {color.fg.lightred}{progress}/{self.max}"                                                  if self.max is not None else ''
        eta      = f" {color.stop}eta {color.fg.blue}{self.update_eta(progress)}"                                  if not stop else ''
        duration = f" {color.fg.purple}{str(datetime.timedelta(seconds=time() - self.start_at)).split('.')[0]}"  if self.duration else ''

        prefix  = '' if prefix == '' else color.stop + prefix + ' '
        suffix  = f"{percent}{frac}{duration}{eta}"

        barwidth = self.width - len(color.clear(suffix)) - len(color.clear(prefix))
        barwidth = max(barwidth,10)

        currentBar = int(round(min(progress_normed*barwidth,barwidth)))
        minBar     = int(min(progress_normed*barwidth,barwidth))

        if progress_normed == 0:   bar = color.fg.lightgrey      + '━' * barwidth
        elif progress_normed == 1: bar = color.fg.lightgreen + '━' * barwidth
        elif currentBar == minBar: bar = c + '━' * currentBar       + color.fg.lightgrey      + '╺' + color.fg.lightgrey  + '━' * (barwidth - currentBar - 1)
        else:                      bar = c + '━' * (currentBar - 1) + c + '╸' + color.fg.lightgrey  + '━' * (barwidth - currentBar)

        msg = f"{prefix}{bar}{suffix}{color.stop}"
        print(msg, end=end)

    def stop(self):
        self(self.lastProgress, stop=True)
        self.lastProgress = 0
        self.lastUpdate = None


from setuptools import setup

requirements = ["gitpython"]
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name="LRFutils",
    version = "0.1.0",
    description = 'Just a custom library to share with some colleagues. Use it at your own risks.',
    author = 'Leirof',
    author_email = 'vince.lrf@gmail.com',
    url = 'https://github.com/LeiRoF/Utils',
    packages=['LRFutils'],
    install_requires=requirements,
    python_requires='>=3.7.0',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers'
    ]
)